/*
 * $Id: eek.c 19 2004-02-22 15:03:15Z tilman $
 *
 * Copyright (C) 2004 Eek project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>

#include "eek.h"
#include "misc.h"
#include "kjofol.h"

static char *theme_type[EEK_THEME_TYPE_NUM] =
	{NULL, "kjofol"};

Eek *eek_new ()
{
	return calloc (1, sizeof (Eek));
}

void eek_free (Eek *e)
{
	if (e)
		free (e);
}

void eek_theme_type_set (Eek *e, char *type)
{
	int i;

	assert (e);
	assert (type);

	e->theme_type = EEK_THEME_TYPE_NONE;

	for (i = 1; i < EEK_THEME_TYPE_NUM; i++)
		if (!strcmp (type, theme_type[i]))
			e->theme_type = i;

	switch (e->theme_type) {
		case EEK_THEME_TYPE_NONE:
			e->convert = NULL;
			break;
		case EEK_THEME_TYPE_KJOFOL:
			e->convert = kjofol_convert;
			break;
		default:
			assert(false);
			break;
	}
}

void eek_source_set (Eek *e, const char *source)
{
	char *ptr;

	assert (e);
	assert (source);

	strlcpy (e->source, source, sizeof (e->source));

	strlcpy (e->source_dir, source, sizeof (e->source_dir));

	if ((ptr = strrchr (e->source_dir, '/')))
		*(ptr--) = 0;
}

const char *eek_source_get (Eek *e)
{
	assert (e);

	return e->source;
}

const char *eek_source_dir_get (Eek *e)
{
	assert (e);

	return e->source_dir;
}

void eek_dest_set (Eek *e, const char *dest)
{
	assert (e);
	assert (dest);

	strlcpy (e->dest, dest, sizeof (e->dest));
}

const char *eek_dest_get (Eek *e)
{
	assert (e);

	return e->dest;
}

void eek_name_set (Eek *e, const char *name)
{
	assert (e);
	assert (name);

	strlcpy (e->name, name, sizeof (e->name));
}

const char *eek_name_get (Eek *e)
{
	assert (e);

	return e->name;
}

static bool copy_skel (Eek *e)
{
	char buf[PATH_MAX + 1], buf2[PATH_MAX + 1];

	assert (e);

	create_dir (e->dest);

	snprintf (buf, sizeof (buf),
	          DATADIR "/theme-skel/%s", theme_type[e->theme_type]);

	if (!copy_tree (buf, e->dest))
		return false;

	snprintf (buf, sizeof (buf), "%s/skel.edc", e->dest);
	snprintf (buf2, sizeof (buf2), "%s/%s.edc", e->dest, e->name);
	rename (buf, buf2);

	return true;
}

bool eek_run (Eek *e)
{
	assert (e);

	assert (e->theme_type);
	assert (*e->source);
	assert (*e->dest);

	if (!copy_skel (e))
		return false;

	return e->convert (e);
}
