/*
 * $Id: kjofol.c 14 2004-02-15 18:11:50Z tilman $
 *
 * Copyright (C) 2004 Eek project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <X11/Xlib.h>
#include <Imlib2.h>
#include <assert.h>
#include <stdio.h>
#include <string.h>

#include "eek.h"
#include "misc.h"
#include "kjofol_theme.h"
#include "kjofol_edc.h"

#define PREPARE_IMAGE(w, h)                                        \
	imlib_image_set_format ("png");                                \
	imlib_image_attach_data_value ("quality", NULL, 99, NULL);     \
	imlib_image_attach_data_value ("compression", NULL, 9, NULL);  \
	fix_alpha ((w), (h));

static void fix_alpha (int w, int h)
{
	DATA32 *data, *ptr, *end;

	data = imlib_image_get_data ();
	assert (data);

	end = data + w * h;

	for (ptr = data; ptr != end; ptr++)
		if (*ptr == 0xffff00ff)
			*ptr = 0xffffff;

	imlib_image_set_has_alpha (true);
	imlib_image_put_back_data (data);
}

static Imlib_Image convert_background (KjTheme *k, int group, int state)
{
	KjThemeElement *e;
	Imlib_Image im;
	int index;
	char path[PATH_MAX + 1];
	char *element[] = {"BackgroundImage", "BackgroundImagePressed1",
	                   "PlaylistBmp", "PlaylistBmpPressed"};

	assert (k);

	index = (group - 1) * 2 + state;

	if (!(e = kjofol_theme_element_find (k, KJ_THEME_ELEMENT_TYPE_IMAGE,
	                                     group, element[index]))) {
		fprintf (stderr, "Cannot find %s!\n", element[index]);
		return NULL;
	}

	/* check whether the theme provides this image */
	if (!*e->data.im.path)
		return NULL;

	snprintf (path, sizeof (path), "%s/%s", eek_source_dir_get (k->e),
	          e->data.im.path);

	if (!(im = imlib_load_image (path)))
		return NULL;

	imlib_context_set_image (im);
	e->data.im.w = imlib_image_get_width ();
	e->data.im.h = imlib_image_get_height ();

	snprintf (path, sizeof (path), "%s/images/%s_%s.png",
	          eek_dest_get (k->e),
	          kjofol_theme_element_group[group],
	          kjofol_theme_element_state[state]);

	PREPARE_IMAGE(e->data.im.w, e->data.im.h);
	imlib_save_image (path);

	return im;
}

static bool convert_buttons (KjTheme *k, Imlib_Image source,
                             int group, int state)
{
	KjThemeElement *e;
	List *l;
	Imlib_Image im;
	char path[PATH_MAX + 1];

	assert (k);
	assert (source);

	for (l = k->elements; l; l = l->next) {
		e = l->data;

		if (e->type != KJ_THEME_ELEMENT_TYPE_BUTTON
		    || e->group != group)
			continue;

		imlib_context_set_image (source);
		im = imlib_create_cropped_image (e->data.but.x,
		                                 e->data.but.y,
		                                 e->data.but.w,
		                                 e->data.but.h);

		imlib_context_set_image (im);

		snprintf (path, sizeof (path), "%s/images/%s_%s.png",
		          eek_dest_get (k->e), e->id,
		          kjofol_theme_element_state[state]);

		PREPARE_IMAGE(e->data.but.w, e->data.but.h);
		imlib_save_image (path);
		imlib_free_image_and_decache ();
	}

	return true;
}

static bool convert_images (KjTheme *k)
{
	Imlib_Image bg;
	int i, j;

	assert (k);

	/* for each ElementGroup */
	for (i = KJ_THEME_ELEMENT_GROUP_MAIN;
	     i < KJ_THEME_ELEMENT_GROUP_NUM; i++) {
		/* for each state... */
		for (j = 0; j < KJ_THEME_ELEMENT_STATE_NUM; j++) {
			if (!(bg = convert_background (k, i, j)))
				continue;

			convert_buttons (k, bg, i, j);

			imlib_context_set_image (bg);
			imlib_free_image_and_decache ();
		}
	}

	return true;
}

static void register_theme_elements (KjTheme *k)
{
	char *button[] = {"PlayButton", "StopButton", "PauseButton",
	                  "ForwardButton", "RewindButton", "RepeatButton",
	                  "CloseButton", "OpenFileButton",
	                  "EqualizerButton", "PlayListButton",
	                  "NextSongButton", "PreviousSongButton",
	                  "ShuffleButton", NULL};
	char *image[] = {"BackgroundImage", "BackgroundImagePressed1",
	                 NULL};
	char *pl_image[] = {"PlaylistBmp", "PlaylistBmpPressed", NULL};
	char *window[] = {"MP3TimeWindow", "FilenameWindow",
	                  "MP3KbpsWindow", "MP3KhzWindow", NULL};
	char **ptr;

	assert (k);

	for (ptr = button; *ptr; ptr++)
		kjofol_theme_element_add (k, KJ_THEME_ELEMENT_TYPE_BUTTON,
		                          KJ_THEME_ELEMENT_GROUP_MAIN, *ptr);

	for (ptr = image; *ptr; ptr++)
		kjofol_theme_element_add (k, KJ_THEME_ELEMENT_TYPE_IMAGE,
		                          KJ_THEME_ELEMENT_GROUP_MAIN, *ptr);

	for (ptr = pl_image; *ptr; ptr++)
		kjofol_theme_element_add (k, KJ_THEME_ELEMENT_TYPE_IMAGE,
		                          KJ_THEME_ELEMENT_GROUP_PLAYLIST,
		                          *ptr);

	for (ptr = window; *ptr; ptr++)
		kjofol_theme_element_add (k, KJ_THEME_ELEMENT_TYPE_WINDOW,
		                          KJ_THEME_ELEMENT_GROUP_MAIN, *ptr);
}

bool kjofol_convert (Eek *e)
{
	KjTheme *k;
	bool ret;

	assert (e);

	/* create a kjofol theme description and tell it about the
	 * images and buttons we care about
	 */
	if (!(k = kjofol_theme_new (e)))
		return false;

	register_theme_elements (k);

	/* now read and parse the kjofol RC file */
	if (!kjofol_theme_read (k)) {
		kjofol_theme_free (k);
		return false;
	}

	/* convert the background images, buttons etc */
	if (!convert_images (k)) {
		kjofol_theme_free (k);
		return false;
	}

	/* now dump the EDC */
	ret = kjofol_edc_write (k);

	kjofol_theme_free (k);

	return ret;
}
