/*
 * $Id: kjofol_edc.c 17 2004-02-15 18:31:04Z tilman $
 *
 * Copyright (C) 2004 Eek project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>

#include "eek.h"
#include "kjofol_theme.h"

static bool write_background (KjTheme *k, FILE *f)
{
	KjThemeElement *e;

	assert (k);
	assert (f);

	/* background image */
	if (!(e = kjofol_theme_element_find (k, KJ_THEME_ELEMENT_TYPE_IMAGE,
	                                     KJ_THEME_ELEMENT_GROUP_MAIN,
	                                     "BackgroundImage")))
		return false;

	fprintf (f, "#define TOTAL_W %i\n", e->data.im.w);
	fprintf (f, "#define TOTAL_H %i\n\n", e->data.im.h);

	return true;
}

static bool write_window (KjTheme *k, FILE *f,
                          KjThemeElementGroup group,
                          char *id, char *prefix)
{
	KjThemeElement *e;
	KjWindow *w;

	assert (k);
	assert (f);

	if (!(e = kjofol_theme_element_find (k, KJ_THEME_ELEMENT_TYPE_WINDOW,
	                                     group, id)))
		return false;

	w = &e->data.win;

	fprintf (f, "#define %s_X1 %i\n", prefix, w->x);
	fprintf (f, "#define %s_Y1 %i\n", prefix, w->y);
	fprintf (f, "#define %s_X2 %i\n", prefix, w->x + w->w);
	fprintf (f, "#define %s_Y2 %i\n", prefix, w->y + w->h);

	/* FIXME */
	fprintf (f, "#define %s_FONT Vera\n", prefix);
	fprintf (f, "#define %s_COLOR 244 210 255 255\n", prefix);
	fprintf (f, "#define %s_SIZE 8\n\n", prefix);

	return true;
}

static bool write_windows (KjTheme *k, FILE *f)
{
	assert (k);
	assert (f);

	if (!(write_window (k, f, KJ_THEME_ELEMENT_GROUP_MAIN,
	                    "FilenameWindow", "TRACK_NAME")))
		return false;

	if (!(write_window (k, f, KJ_THEME_ELEMENT_GROUP_MAIN,
	                    "MP3TimeWindow", "TIME_TEXT")))
		return false;

	if (!(write_window (k, f, KJ_THEME_ELEMENT_GROUP_MAIN,
	                    "MP3KbpsWindow", "TRACK_BITRATE")))
		return false;

	if (!(write_window (k, f, KJ_THEME_ELEMENT_GROUP_MAIN,
	                    "MP3KhzWindow", "TRACK_SAMPLE_RATE")))
		return false;

	return true;
}

static bool write_buttons (KjTheme *k, FILE *f)
{
	KjThemeElement *e;
	KjButton *b;
	List *l;
	char id[64], *src, *dest;

	assert (k);
	assert (f);

	for (l = k->elements; l; l = l->next) {
		e = l->data;

		if (e->type != KJ_THEME_ELEMENT_TYPE_BUTTON)
			continue;

		b = &e->data.but;
		dest = id;
		src = e->id;

		while ((*dest++ = toupper (*src++)));

		fprintf (f, "#define %s_W %i\n", id, b->w);
		fprintf (f, "#define %s_H %i\n", id, b->h);
		fprintf (f, "#define %s_X1 %i\n", id, b->x);
		fprintf (f, "#define %s_Y1 %i\n", id, b->y);
		fprintf (f, "#define %s_X2 %i\n", id, b->x + b->w);
		fprintf (f, "#define %s_Y2 %i\n\n", id, b->y + b->h);
	}

	return true;
}

static bool write_settings (KjTheme *k)
{
	FILE *f;
	char file[PATH_MAX + 1];

	assert (k);

	snprintf (file, sizeof (file), "%s/settings.inc",
	          eek_dest_get (k->e));

	if (!(f = fopen (file, "w")))
		return false;

	if (!write_background (k, f)) {
		fclose (f);
		return false;
	}

	if (!write_windows (k, f)) {
		fclose (f);
		return false;
	}

	if (!write_buttons (k, f)) {
		fclose (f);
		return false;
	}

	return true;
}

static bool write_makefile (KjTheme *k)
{
	FILE *f;
	char file[PATH_MAX + 1];

	assert (k);

	snprintf (file, sizeof (file), "%s/Makefile",
	          eek_dest_get (k->e));

	if (!(f = fopen (file, "w")))
		return false;

	fprintf (f, "%s.eet: %s.edc settings.inc\n",
	         k->e->name, k->e->name);
	fprintf (f, "\tedje_cc -v -id images -fd fonts %s.edc\n",
	         k->e->name);
	fprintf (f, "\nclean:\n\trm -f %s.eet\n", k->e->name);
	fclose (f);

	return true;
}

bool kjofol_edc_write (KjTheme *k)
{
	assert (k);

	if (!write_settings (k))
		return false;

	if (!write_makefile (k))
		return false;

	return true;
}
