/*
 * $Id: kjofol_theme.c 19 2004-02-22 15:03:15Z tilman $
 *
 * Copyright (C) 2004 Eek project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>

#include "eek.h"
#include "misc.h"
#include "kjofol_theme.h"

#define IS_IDENT(str, v)                    \
	(!strncmp ((str), (v)->id, (v)->id_len) \
	 && (str)[(v)->id_len] == ' ')

#define READ_IMAGE(dest, src)    \
	strlcpy ((dest).path, src, sizeof ((dest).path))

#define READ_GEOM(dest, src) {                           \
	sscanf ((src), "%i %i %i %i",                        \
	        &(dest).x, &(dest).y, &(dest).w, &(dest).h); \
	(dest).w -= (dest).x;                                \
	(dest).h -= (dest).y;                                \
}

static bool read_rc_file (KjTheme *k, const char *file);

const char *kjofol_theme_element_state[KJ_THEME_ELEMENT_STATE_NUM] =
	{"default", "pressed"};
const char *kjofol_theme_element_group[KJ_THEME_ELEMENT_GROUP_NUM] =
	{NULL, "background", "playlist"};

static void parse_line (KjTheme *k, char *buf)
{
	KjThemeElement *e;
	List *l;
	const char *include = "IncludeRCFile";
	char path[PATH_MAX + 1];
	int include_len = strlen (include);

	for (l = k->elements; l; l = l->next) {
		e = l->data;

		if (!IS_IDENT (buf, e))
			continue;

		switch (e->type) {
			case KJ_THEME_ELEMENT_TYPE_IMAGE:
				READ_IMAGE (e->data.im, &buf[e->id_len + 1]);
				break;
			case KJ_THEME_ELEMENT_TYPE_BUTTON:
				READ_GEOM (e->data.but, &buf[e->id_len + 1]);
				break;
			case KJ_THEME_ELEMENT_TYPE_WINDOW:
				READ_GEOM (e->data.win, &buf[e->id_len + 1]);
				break;
			default:
				assert (false);
				break;
		}
	}

	/* we may have to include another RC file... */
	if (!strncmp (buf, include, include_len)
	    && buf[include_len] == ' ') {
		snprintf (path, sizeof (path), "%s/%s",
		          eek_source_dir_get (k->e), &buf[include_len + 1]);
		read_rc_file (k, path);
	}
}

static bool read_rc_file (KjTheme *k, const char *file)
{
	FILE *fp;
	char buf[1024];

	assert (k);
	assert (file);

	if (!(fp = fopen (file, "r")))
		return false;

	while ((fgets (buf, sizeof (buf), fp))) {
		strstrip (buf);
		parse_line (k, buf);
	}

	fclose (fp);

	/* FIXME
	 * add sanity checks, make sure the important
	 * buttons have been found etc
	 */

	return true;
}

bool kjofol_theme_read (KjTheme *k)
{
	return read_rc_file (k, eek_source_get (k->e));
}

bool kjofol_theme_element_add (KjTheme *k, KjThemeElementType type,
                               KjThemeElementGroup group, char *id)
{
	KjThemeElement *e;

	assert (k);
	assert (type);
	assert (group);
	assert (id);

	if (!(e = calloc (1, sizeof (KjThemeElement))))
		return false;

	strlcpy (e->id, id, sizeof (e->id));
	e->id_len = strlen (id);
	e->type = type;
	e->group = group;

	k->elements = list_prepend (k->elements, e);

	return true;
}

KjThemeElement *kjofol_theme_element_find (KjTheme *k,
                                           KjThemeElementType type,
                                           KjThemeElementGroup group,
                                           char *id)
{
	KjThemeElement *e;
	List *l;

	assert (k);
	assert (type);
	assert (group);
	assert (id);

	for (l = k->elements; l; l = l->next) {
		e = l->data;

		if (e->type == type
		    && e->group == group
		    && !strcmp (e->id, id))
			return e;
	}

	return NULL;
}

KjTheme *kjofol_theme_new (Eek *e)
{
	KjTheme *k;

	assert (e);

	if (!(k = calloc (1, sizeof (KjTheme))))
		return NULL;

	k->e = e;

	return k;
}

void kjofol_theme_free (KjTheme *k)
{
	KjThemeElement *e;

	if (!k)
		return;

	while (k->elements) {
		e = k->elements->data;
		k->elements = list_remove (k->elements, e);
		free (e);
	}

	free (k);
}
