/*
 * $Id: main.c 19 2004-02-22 15:03:15Z tilman $
 *
 * Copyright (C) 2004 Eek project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <stdio.h>
#include <getopt.h>

#include "eek.h"

static void show_usage ()
{
	printf ("\nUsage: " PACKAGE " options\n\n"
	        "Options:\n"
	        "    -t, --type=...    specify theme type\n"
	        "    -n, --name=...    specify theme name\n"
	        "    -s, --source=...  specify original theme source\n"
	        "    -d, --dest=...    specify new theme destination dir\n\n");
}

static bool handle_args (Eek *e, int argc, char **argv)
{
	int o;
	struct option opts[] = {{"help", no_argument, NULL, 'h'},
	                        {"type", required_argument, NULL, 't'},
	                        {"name", required_argument, NULL, 'n'},
	                        {"source", required_argument, NULL, 's'},
	                        {"dest", required_argument, NULL, 'd'},
	                        {NULL, 0, NULL, 0}};

	while ((o = getopt_long (argc, argv, "ht:n:s:d:", opts, NULL)) != -1) {
		switch (o) {
			case 'h':
				return false;
				break;
			case 't':
				eek_theme_type_set (e, optarg);
				break;
			case 'n':
				eek_name_set (e, optarg);
				break;
			case 's':
				eek_source_set (e, optarg);
				break;
			case 'd':
				eek_dest_set (e, optarg);
				break;
			default:
				break;
		}
	}

	return true;
}

int main (int argc, char **argv)
{
	Eek *e;
	int ret;

	if (!(e = eek_new ()))
		return 1;

	if (!handle_args (e, argc, argv)
	    || !*e->source || !*e->dest || !*e->name || !e->theme_type) {
		show_usage ();
		eek_free (e);
		return 1;
	}

	ret = eek_run (e);

	eek_free (e);

	return ret;
}
