/*
 * $Id: misc.c 19 2004-02-22 15:03:15Z tilman $
 *
 * Copyright (C) 2004 Eek project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <limits.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>

bool is_dir (const char *dir)
{
	struct stat st;

	if (stat (dir, &st))
		return false;

	return (S_ISDIR(st.st_mode));
}

char *cut_suffix (char *in)
{
	char *ptr;

	assert (in);

	if ((ptr = strrchr (in, '.')))
		*ptr = 0;

	return in;
}

/*
 * This code was copied from OpenBSD's C library (see
 * ftp://ftp.openbsd.org/pub/OpenBSD/src/lib/libc/string/strlcpy.c or
 * alternatively strlcat.c) with minor formatting corrections.
 *
 * Copyright (c) 1998, 2000 Todd C. Miller <Todd.Miller@courtesan.com>
 */
#ifndef HAVE_STRLCPY
size_t strlcpy (char *dst, const char *src, size_t siz)
{
	char *d = dst;
	const char *s = src;
	size_t n = siz;

	/* Copy as many bytes as will fit */
	if (n && --n) {
		do {
			if (!(*d++ = *s++))
				break;
		} while (--n);
	}

	/* Not enough room in dst, add NULL and traverse rest of src */
	if (!n) {
		if (siz)
			*d = 0; /* NUL-terminate dst */

		while (*s++);
	}

	return (s - src - 1); /* count does not include NUL */
}
#endif

bool create_dir (const char *dir)
{
	char buf[PATH_MAX + 10];

	assert (dir);

	if (is_dir (dir))
		return true;

	/* FIXME: ugh, don't use system()! */
	snprintf (buf, sizeof (buf), "mkdir -p %s", dir);

	if (system (buf))
		return false;

	return is_dir (dir);
}

bool copy_tree (const char *src, const char *dest)
{
	char buf[PATH_MAX + 10];

	assert (src);
	assert (dest);

	if (!is_dir (src))
		return false;

	/* FIXME: ugh, don't use system()! */
	snprintf (buf, sizeof (buf), "cp -R %s/* %s", src, dest);

	return !system (buf);
}

/**
 * Remove leading and trailing whitespace
 *
 * @param str
 * @return Trimmed string.
 */
char *strstrip (char *str)
{
	char *start, *ptr = str;

	assert (str);

	if (!strlen (str))
		return str;

	/* step over leading whitespace */
	for (start = str; isspace (*start); start++);

	if (str != start) {
		while ((*ptr++ = *start++));
		*ptr = 0;
	}

	if (!strlen (str))
		return str;

	ptr = &str[strlen (str) - 1];

	if (!isspace (*ptr))
		return str;

	while (isspace (*ptr) && ptr > str)
		ptr--;

	ptr[1] = 0;

	return str;
}
